@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: paned_notebook.c,v 1.23.2.1 2004/05/01 04:34:27 fuyu Exp $
+ * $Id: paned_notebook.c,v 1.23.2.5 2004/05/04 13:52:44 fuyu Exp $
  */
 
 #define GTK_COMPILATION	1	/* 
@@ -190,12 +190,13 @@ enum {
   ITEM_VIEW_BEING_CLOSED_SIGNAL,
   ITEM_VIEW_CLOSED_SIGNAL,
   PAGE_SWITCHED_SIGNAL,
+  PAGE_DOUBLE_CLICKED_SIGNAL,
   LAST_SIGNAL
 };
 
 
 static GtkPanedClass *parent_class = NULL;
-static int paned_notebook_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
+static int paned_notebook_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0 };
 
 
 enum {
@@ -293,10 +294,23 @@ paned_notebook_class_init(PanedNotebookClass *klass)
 		 G_TYPE_NONE, 2,
 		 GTK_TYPE_WIDGET,
 		 GTK_TYPE_WIDGET);
+  paned_notebook_signals[PAGE_DOUBLE_CLICKED_SIGNAL] =
+    g_signal_new("page_double_clicked",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(PanedNotebookClass, page_double_clicked),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__POINTER_OBJECT,
+		 G_TYPE_NONE, 2,
+		 G_TYPE_POINTER,
+		 GTK_TYPE_WIDGET);
 
   klass->item_view_required = item_view_required_cb;
+  klass->item_view_accepted = NULL;
   klass->item_view_being_closed = NULL;
   klass->item_view_closed = NULL;
+  klass->page_switched = NULL;
+  klass->page_double_clicked = NULL;
 }
 
 
@@ -584,7 +598,9 @@ paned_notebook_init(PanedNotebook *paned_notebook)
   gtk_toolbar_insert(paned_notebook->toolbar, tmp_item, -1);
 #endif
 
+#if 0	/* GTK+-2.4.1ǤȤʤΤǤ */
   gtk_widget_set_sensitive(GTK_WIDGET(paned_notebook->contents_menu), FALSE);
+#endif
 
   paned_notebook->item_contents = MOD_NOTEBOOK(mod_notebook_new());
   mod_notebook_set_scrollable(paned_notebook->item_contents, TRUE);
@@ -1558,8 +1574,10 @@ paned_notebook_open_item_view(PanedNotebook *paned_notebook, gpointer item,
 		     GTK_WIDGET(paned_notebook->contents_pane));
     }
 
-  if (paned_notebook->number_of_items_shown == 0)
+#if 0	/* GTK+-2.4.1ǤȤʤΤǤ */
+  if (paned_notebook->number_of_items_shown >= 0)
     gtk_widget_set_sensitive(GTK_WIDGET(paned_notebook->contents_menu), TRUE);
+#endif
 
   if (paned_notebook->number_of_items_shown++ == 0 || in_tab)
     {
@@ -1742,8 +1760,10 @@ close_nth_page(PanedNotebook *paned_notebook, int page)
 
   if (paned_notebook->number_of_items_shown == 0)
     {
+#if 0	/* GTK+-2.4.1ǤȤʤΤǤ */
       gtk_widget_set_sensitive(GTK_WIDGET(paned_notebook->contents_menu),
 			       FALSE);
+#endif
       if (!paned_notebook->always_show_contents_pane)
 	{
 	  if (gtk_paned_get_position(GTK_PANED(paned_notebook)) == 0)
@@ -1837,18 +1857,57 @@ static gboolean
 button_press_event_cb(ModNotebook *notebook, GdkEventButton *event,
 		      PanedNotebook *paned_notebook)
 {
+  int page;
   g_return_val_if_fail(notebook == paned_notebook->item_contents, FALSE);
 
   if (event->button == 2)
     {
-      int page = mod_notebook_get_mouse_event_page_num(notebook,
-							(GdkEventAny*)event);
+      page = mod_notebook_get_mouse_event_page_num(notebook,
+						   (GdkEventAny*)event);
       if (page != -1)
 	{
 	  close_nth_page(paned_notebook, page);
 	  return TRUE;
 	}
     }
+  else if (event->button == 1)
+    {
+      if (event->type == GDK_2BUTTON_PRESS)
+	{
+	  /* ZZZ: ֥륯å٥ */
+	  page = mod_notebook_get_mouse_event_page_num(notebook,
+						       (GdkEventAny*)event);
+	  if (page != -1)
+	    {
+	      ItemViewRelation *relation;
+#if GTK_MINOR_VERSION > 2
+	      ItemViewRelation tmp_relation = { NULL, NULL, NULL, NULL };
+#endif
+	      GtkWidget *widget
+		= mod_notebook_get_nth_page(paned_notebook->item_contents,
+					    page);
+	      if (widget == NULL)
+		return TRUE;
+
+#if GTK_MINOR_VERSION <= 2
+	      relation = find_item_view_relation(paned_notebook, widget);
+#else
+	      relation = find_item_view_relation(paned_notebook, widget,
+						 &tmp_relation);
+#endif
+	      if (relation == NULL || relation->item_view != widget)
+		return TRUE;
+
+	      g_signal_emit(G_OBJECT(paned_notebook),
+			    paned_notebook_signals[PAGE_DOUBLE_CLICKED_SIGNAL],
+			    0,
+			    relation->item,
+			    relation->item_view);
+
+	      return FALSE;
+	    }
+	}
+    }
 
   return FALSE;
 }
